-------------------------------------------------------------------------------
--
-- Design unit:   Fabric registers available from MSS by AHB bus
--                 
--
-- File name:     ahb_reg.vhd
--
-- Description:   This block makes access to fabric registers from MSS. It uses
--                AHB slave interface for this purpose. New registers can 
--                be easily added to block, but MSS firmaware must be remade.
--
--                It consists of following registers:
--
--                cmd_reg - command register is directly accessible by Fab 
--                packet (0xFB prefix). If new value is written to cmd_reg, 
--                then new_cmd flag is asserted. Commands allows to create 
--                various operations in FPGA fabric.
--
--                state_reg - state register provides information about 
--                the FPGA for a MSS and subsequently to the host PC. 
--
--                stage_reg - stage register shows actual address of RAM 
--                processing for MSS system. 
--
--                fabin_reg, fabout_reg - both registers are directly available 
--                by Fab packet (0xFB prefix). fabin_reg is fabric input register
--                (from MSS to FPGA), it is 32-bit payload value of Fab packet
--                and fabout_reg or fabric output register(from FPGA to MSS) is 
--                sent as payload of fabric status packet. Fabric status packet 
--                (prefix 0xF5)is always sent as response to Fab packet.
-- 
--                fabst_reg - this register contains 32 status bit word, which 
--                can be used to monitor internal FPGA state. It is inserted
--                in a fabric status packet (prefix 0xF5).
--
--                wstart_reg, wsize_reg are parameters where will be saved data 
--                for user file (data in RAM for MSS file). wstart_reg is start 
--                address of data stream and wsize_reg is size of stream. Both 
--                registers are configurable only by MSS.
--
--                rstart_reg, rsize_reg are parameters where data of user file was 
--                written. Similarly, rstart_reg is start address of stream and 
--                rsize_reg is size of stream. 
--
--                user1_reg, user2_reg - example registers, they are not used. 
--
-- System:        VHDL'93
--
-- Author:        Marek Laban, MICRONIC a.s., LabHC
--
-- Copyright:     MICRONIC a.s., LabHC
--
-- Created in frame of HECTOR project (No. 644052).
--
-- Revision:      Version 1.00, May 2016
-- History:       16/05/2016 - v1.00 Created (ML, MIC)
--
-------------------------------------------------------------------------------


LIBRARY IEEE;

USE IEEE.std_logic_1164.ALL;

ENTITY ahb_reg IS
  PORT (
  -- global signals -----------------------------------------------------------
    n_rst      : IN    STD_LOGIC;                  
    clk        : IN    STD_LOGIC;

  -- AHB SLAVE bus (MSS)-------------------------------------------------------
    haddr_mss  : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
    hwdata_mss : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
    hrdata_mss : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
    hsel_mss   : IN  STD_LOGIC;   
    hready_mss : OUT STD_LOGIC;   
    hwrite_mss : IN  STD_LOGIC;
  -----------------------------------------------------------------------------
    
  -- registers available from MSS
    cmd_reg    : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);  -- command register
    state_reg  : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);  -- state of operation
    stage_reg  : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);  -- actual used address
    --
    fabin_reg  : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);  -- fabric input (from MSS)
    fabout_reg : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);  -- fabric output (to MSS)
    fabst_reg  : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);  -- fabric status (to MSS)
    --
    -- parameters where will be saved data for user file generation (RAM to MSS
    -- file)
    wstart_reg : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);  -- start address
    wsize_reg  : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);  -- size in bytes
    --
    -- parameters where will be loaded user file (MSS file to RAM)
    rstart_reg : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);  -- start address
    rsize_reg  : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);  -- size in bytes
    --
    user1_reg  : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);  -- example registers
    user2_reg  : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);   
    --
  -- new command received signal (assert high)
    new_cmd    : OUT STD_LOGIC
  );
END ahb_reg;

ARCHITECTURE ahb_reg OF ahb_reg IS
  -- MSS address of fabric registers -------------------------------------------   
    -- command register address
    CONSTANT CMD_ADDR      : STD_LOGIC_VECTOR(31 DOWNTO 0) := x"70000000"; 
    -- write to external RAM - first address
    CONSTANT STATE_ADDR    : STD_LOGIC_VECTOR(31 DOWNTO 0) := x"70000004"; 
    -- stage of acquistion (ext. RAM addr.) register address
    CONSTANT STAGE_ADDR    : STD_LOGIC_VECTOR(31 DOWNTO 0) := x"70000008";
    -- fabric input register address 
    CONSTANT FABIN_ADDR    : STD_LOGIC_VECTOR(31 DOWNTO 0) := x"7000000C";
    -- fabric output regiter address
    CONSTANT FABOUT_ADDR   : STD_LOGIC_VECTOR(31 DOWNTO 0) := x"70000010";
    -- fabric output status 
    CONSTANT FABST_ADDR    : STD_LOGIC_VECTOR(31 DOWNTO 0) := x"70000014";
    --
    -- read from external RAM - start address
    CONSTANT WSTART_ADDR   : STD_LOGIC_VECTOR(31 DOWNTO 0) := x"70000018"; 
    -- write to external RAM - size of stream
    CONSTANT WSIZE_ADDR    : STD_LOGIC_VECTOR(31 DOWNTO 0) := x"7000001C"; 
    -- read from external RAM - start address
    CONSTANT RSTART_ADDR   : STD_LOGIC_VECTOR(31 DOWNTO 0) := x"70000020"; 
    -- read from external RAM - size of stream
    CONSTANT RSIZE_ADDR    : STD_LOGIC_VECTOR(31 DOWNTO 0) := x"70000024"; 
    --
    -- user register 1 
    CONSTANT USER1_ADDR    : STD_LOGIC_VECTOR(31 DOWNTO 0) := x"70000028";   
    -- user register 2
    CONSTANT USER2_ADDR    : STD_LOGIC_VECTOR(31 DOWNTO 0) := x"7000002C";   

  -- states and registers of MSS AHB bus communication -------------------------
    -- ahb state type definition
    TYPE ahb_state_states IS (  AHB_ADDR,
                                AHB_DATA_WR,
                                AHB_DATA_RD
                             ); 
    -- ahb state definition
    SIGNAL ahb_state        : ahb_state_states;
    -- internal address of AHB implementation
    SIGNAL int_addr         : STD_LOGIC_VECTOR(31 DOWNTO 0);
    --    
  -- internal registers of fabric ----------------------------------------------
    -- command register
    SIGNAL cmd_reg_i    : STD_LOGIC_VECTOR(31 DOWNTO 0);  -- command register
    --
    SIGNAL fabin_reg_i  : STD_LOGIC_VECTOR(31 DOWNTO 0);  -- fabric input
    -- registers of external RAM operations
    SIGNAL wstart_reg_i : STD_LOGIC_VECTOR(31 DOWNTO 0);  -- write start address
    SIGNAL wsize_reg_i  : STD_LOGIC_VECTOR(31 DOWNTO 0);  -- write size in bytes
    --
    SIGNAL rstart_reg_i : STD_LOGIC_VECTOR(31 DOWNTO 0);  -- read start address
    SIGNAL rsize_reg_i  : STD_LOGIC_VECTOR(31 DOWNTO 0);  -- read size in bytes
    --
    SIGNAL user1_reg_i  : STD_LOGIC_VECTOR(31 DOWNTO 0);  -- read size in bytes
BEGIN
 
-- AHB bus communication -------------------------------------------------------
  PROCESS(clk, n_rst)
  BEGIN
    IF (n_rst = '0') THEN
      ahb_state    <= AHB_ADDR;         -- set default state of AHB bus
      cmd_reg_i    <= (OTHERS =>'0');
      fabin_reg_i  <= (OTHERS =>'0');
      wstart_reg_i <= x"A0000000";      -- set default DMA addresses
      wsize_reg_i  <= x"00000200";
      rstart_reg_i <= x"A0000000";    
      rsize_reg_i  <= x"00000200";    
      user1_reg_i  <= (OTHERS =>'0');
      hready_mss   <= '0';
      hrdata_mss   <= (OTHERS =>'0'); 
    ELSIF rising_edge(clk) THEN
      CASE ahb_state IS                 
        WHEN AHB_ADDR =>              -- address phase
          hready_mss <= '0';            
          -- if hsel_mss active, save address
          IF    (hsel_mss = '1' AND hwrite_mss = '1') THEN        
            ahb_state <= AHB_DATA_WR;
          --
          -- if hsel_mss active, save address
          ELSIF (hsel_mss = '1' AND hwrite_mss = '0') THEN        
            ahb_state <= AHB_DATA_RD;
          END IF;
          int_addr  <= haddr_mss;
        WHEN AHB_DATA_WR =>           -- data phase - write data from MSS
          CASE int_addr IS            -- choose register by address
            WHEN CMD_ADDR   =>                 
              cmd_reg_i      <= hwdata_mss;
            WHEN FABIN_ADDR =>
              fabin_reg_i    <= hwdata_mss;
            WHEN WSTART_ADDR =>              
              wstart_reg_i   <= hwdata_mss;
            WHEN WSIZE_ADDR  =>              
              wsize_reg_i    <= hwdata_mss;
            WHEN RSTART_ADDR =>              
              rstart_reg_i   <= hwdata_mss;
            WHEN RSIZE_ADDR  =>              
              rsize_reg_i    <= hwdata_mss;
            WHEN USER1_ADDR  =>              
              user1_reg_i    <= hwdata_mss;
            WHEN OTHERS => NULL;
          END CASE;
          hready_mss    <= '1';  -- after accessing to register assert hready 
          ahb_state <= AHB_ADDR; -- go back to address phase
          --
        WHEN AHB_DATA_RD =>      -- data phase - read data from MSS
          CASE int_addr IS       -- choose register by address
            WHEN CMD_ADDR    =>                 
              hrdata_mss     <= cmd_reg_i;
            WHEN STATE_ADDR  =>              
              hrdata_mss     <= state_reg;
            WHEN STAGE_ADDR  =>              
              hrdata_mss     <= stage_reg;
            WHEN FABIN_ADDR =>
              hrdata_mss     <= fabin_reg_i;
            WHEN FABOUT_ADDR =>
              hrdata_mss     <= fabout_reg;
            WHEN FABST_ADDR  =>
              hrdata_mss     <= fabst_reg;
            WHEN WSTART_ADDR =>              
              hrdata_mss     <= wstart_reg_i;
            WHEN WSIZE_ADDR  =>                   
              hrdata_mss     <= wsize_reg_i;
            WHEN RSTART_ADDR =>              
              hrdata_mss     <= rstart_reg_i;
            WHEN RSIZE_ADDR  =>                   
              hrdata_mss     <= rsize_reg_i;
            WHEN USER1_ADDR  =>                   
              hrdata_mss     <= user1_reg_i;
            WHEN USER2_ADDR  =>                   
              hrdata_mss     <= user2_reg;
            WHEN OTHERS => 
              hrdata_mss     <= (OTHERS =>'0');
          END CASE;
          hready_mss    <= '1';   -- after accessing to register assert hready 
          ahb_state <= AHB_ADDR;  -- go back to address phase
        WHEN OTHERS => NULL;
      END CASE;
    END IF;
  END PROCESS;
 
  new_cmd   <= '1' WHEN (ahb_state  = AHB_DATA_WR AND 
                         int_addr   = CMD_ADDR    ) ELSE '0';

  cmd_reg    <= cmd_reg_i;
  fabin_reg  <= fabin_reg_i;
  wstart_reg <= wstart_reg_i;
  wsize_reg  <= wsize_reg_i;
  rstart_reg <= rstart_reg_i;
  rsize_reg  <= rsize_reg_i;
  user1_reg  <= user1_reg_i;


end ahb_reg;
