-------------------------------------------------------------------------------
--
-- Design unit:   Control block
--                 
--
-- File name:     ctrl.vhd
--
-- Description:   Control block is fundamental block of HECTOR evaluation 
--                platform, it consists of two components: ahb_reg and ram_if. 
--                Ahb_reg operates registers which are available from MSS. 
--                Ram_if ensure data transfer from/to external RAM. Control 
--                block uses these two components to operate with user entity 
--                and transfer data to PC host. A lot of signals and registers 
--                are not used, they are used only as a template for making 
--                user adjustable system. 
--
--                This user adjustable system consists of two tools, the first 
--                one is a command execution state machine (CMD_EXE) and the 
--                second one is user application logic. 
--
--                CMD_EXE executes command when new data are written to the CMD
--                register. Commands can be triggered by Fab packet (packet 
--                writes data to command and fabin register). New command can be 
--                added by creating of new command definition (constant) and its 
--                inserting to CMD_EXE.
--
--                Example is designed to acquire ouput of TRNG in external 
--                daughter board. It collects 32-bit words from s_p converter and 
--                save it to the external RAM memory.
-- 
--
-- System:        VHDL'93
--
-- Author:        Marek Laban, MICRONIC a.s., LabHC
--
-- Copyright:     MICRONIC a.s., LabHC
--
-- Created in frame of HECTOR project (No. 644052).
--
-- Revision:      Version 1.00, May 2016
-- History:       16/05/2016 - v1.00 Created (ML, MIC)
--
-------------------------------------------------------------------------------



LIBRARY IEEE;

USE ieee.std_logic_unsigned.ALL;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_arith.ALL;

ENTITY ctrl IS
  PORT (
  -----------------------------------------------------------------------------
  -- AHB MASTER bus (external RAM) --------------------------------------------
  -----------------------------------------------------------------------------
    htrans     : OUT STD_LOGIC_VECTOR(1  DOWNTO 0);   
    hsize      : OUT STD_LOGIC_VECTOR(1  DOWNTO 0);
    hwrite     : OUT STD_LOGIC;    
    hwdata     : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);   
    haddr      : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
    hmastlock  : OUT STD_LOGIC;            
    hsel       : OUT STD_LOGIC;  
    hreadyout  : IN  STD_LOGIC; 
    hresp      : IN  STD_LOGIC;       
    hrdata     : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
    hburst     : OUT STD_LOGIC_VECTOR(2  DOWNTO 0);
    --
  -----------------------------------------------------------------------------
  -- AHB SLAVE bus (MSS)-------------------------------------------------------
  -----------------------------------------------------------------------------
    haddr_mss  : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
    hwdata_mss : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
    hrdata_mss : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
    hsel_mss   : IN  STD_LOGIC;   
    hready_mss : OUT STD_LOGIC;   
    hwrite_mss : IN  STD_LOGIC;
    --
  -----------------------------------------------------------------------------
  -- Control block signals and registers --------------------------------------
  -----------------------------------------------------------------------------
  -- global signals 
    n_rst      : IN  STD_LOGIC;
    clk        : IN  STD_LOGIC;
  -- S/P signals
    sp_data    : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);  -- data from SP
    sp_data_vld: IN  STD_LOGIC;                      -- data valid from SP
    sync_en    : OUT STD_LOGIC;                      -- synchronization output
  -- multiplexor out (0 - HDMI,1 - SATA)
    db_mux_out : OUT STD_LOGIC
 
  );
END ctrl;

ARCHITECTURE architecture_ctrl OF ctrl IS

  -- BEGIN COMPONENT DEFINITIONS **********************************************

  COMPONENT ram_if IS
  PORT(
  -- global reset and clock signals
    n_rst            : IN    STD_LOGIC;                  
    clk              : IN    STD_LOGIC;                  
  ---------------------------------------------------------------------------- 
  -- AHB bus signals ---------------------------------------------------------
  ----------------------------------------------------------------------------
    -- AHB-Lite transfer type for master. 
    -- Indicates the type of the current transfer
    htrans            : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);   
    -- AHB-Lite transfer size for master. Indicates the size of the transfer. 
    -- Only byte, half-word and word transactions are supported
    hsize             : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);
    -- AHB-Lite write indication for master 0. 
    -- High for a write, Low for a read.           
    hwrite            : OUT   STD_LOGIC;    
    -- AHB-Lite write data for master                   
    hwdata            : OUT   STD_LOGIC_VECTOR(31 DOWNTO 0);   
    -- AHB-Lite address
    haddr             : OUT   STD_LOGIC_VECTOR(31 DOWNTO 0);
    -- AHB-Lite locked sequence indication for master 0      
    hmastlock         : OUT   STD_LOGIC;            
    -- AHB-Lite slave 0 select                     
    hsel              : OUT   STD_LOGIC;  
    -- AHB-Lite ready signal from slave 0                    
    hreadyout         : IN    STD_LOGIC; 
    -- AHB-Lite transfer response for master                     
    hresp             : IN    STD_LOGIC;       
    -- AHB-Lite read data for master              
    hrdata            : IN    STD_LOGIC_VECTOR(31 DOWNTO 0);
    -- AHB-Lite burst register output
    hburst            : OUT   STD_LOGIC_VECTOR(2 DOWNTO 0);
    --
  ----------------------------------------------------------------------------
  -- Interface for R/W data to/from external RAM memmory ---------------------
  ----------------------------------------------------------------------------
    -- data input and valid input data signal
    data_in           : IN    STD_LOGIC_VECTOR(31 DOWNTO 0);
    data_in_vld       : IN    STD_LOGIC;  -- data are valid during asserting 1 
    -- data output and valid output data signal
    data_out          : OUT   STD_LOGIC_VECTOR(31 DOWNTO 0);
    data_out_vld      : OUT   STD_LOGIC;  -- data are valid during asserting 1
    --
    -- start address of DMA 
    addr_in           : IN    STD_LOGIC_VECTOR(31 DOWNTO 0);
    -- bytes count (number of 8-bit words (rounded to multiple of 4))
    byte_cnt          : IN    STD_LOGIC_VECTOR(31 DOWNTO 0);
    --
    -- control signals  
    data_wr           : IN    STD_LOGIC;  -- data read(0) or data write(1) 
    ram_strt          : IN    STD_LOGIC;  -- ram start (1 during one clk period)
    -- info signals  
    wr_buf_fl         : OUT   STD_LOGIC;  -- write buffer full (1 when full)
    buff_ofw          : OUT   STD_LOGIC;  -- buffer overflow (1 when overflow)
    ram_bsy           : OUT   STD_LOGIC;  -- ram busy (1 when busy, 0 when idle) 
    -- actual address of ram opertion
    ram_stg           : OUT   STD_LOGIC_VECTOR(31 DOWNTO 0) 
  );
  END COMPONENT;
  
  COMPONENT ahb_reg IS
  PORT (
  -- global signals 
    n_rst      : IN    STD_LOGIC;                  
    clk        : IN    STD_LOGIC;
    --
  -- AHB SLAVE bus (MSS)
    haddr_mss  : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
    hwdata_mss : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
    hrdata_mss : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
    hsel_mss   : IN  STD_LOGIC;   
    hready_mss : OUT STD_LOGIC;   
    hwrite_mss : IN  STD_LOGIC;
    -- 
  -- registers available from MSS
    cmd_reg    : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);  -- command register
    state_reg  : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);  -- state of operation
    stage_reg  : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);  -- actual used address
    --
    fabin_reg  : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);  -- fabric input (from MSS)
    fabout_reg : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);  -- fabric output (to MSS)
    fabst_reg  : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);  -- fabric status (to MSS)
    --
    -- parameters where will be saved data for user file generation 
    -- (RAM to MSS file)
    wstart_reg : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);  -- start address
    wsize_reg  : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);  -- size in bytes
    --
    -- parameters where will be loaded user file (MSS file to RAM)
    rstart_reg : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);  -- start address
    rsize_reg  : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);  -- size in bytes
    --
    user1_reg  : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);  -- user optional registers
    user2_reg  : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);   
    --
  -- new command received signal (assert high)
    new_cmd    : OUT STD_LOGIC
  -----------------------------------------------------------------------------
  );
  END COMPONENT;
  -- END COMPONENT DEFINITIONS ************************************************


  -- BEGIN COMMAND ************************************************************
  -- command definitions for the control block  
    -- Idle mode (nothing to do)
    CONSTANT CMD_OP_IDLE    : STD_LOGIC_VECTOR(31 DOWNTO 0) := x"00000000";
    -- Operation start !! fixed for MSS !!
    CONSTANT CMD_OP_ST      : STD_LOGIC_VECTOR(31 DOWNTO 0) := x"00000001";
    -- Set daughter board 1 active (HDMI) 
    CONSTANT CMD_SETDB1     : STD_LOGIC_VECTOR(31 DOWNTO 0) := x"00000002";
    -- Set daughter board 2 active (SATA) 
    CONSTANT CMD_SETDB2     : STD_LOGIC_VECTOR(31 DOWNTO 0) := x"00000003"; 
    -- Enable SP converter synchronization
    CONSTANT CMD_ENSYNC     : STD_LOGIC_VECTOR(31 DOWNTO 0) := x"00000004";
    -- Disable SP converter synchronization
    CONSTANT CMD_DSSYNC     : STD_LOGIC_VECTOR(31 DOWNTO 0) := x"00000005"; 
    -- User command 0 (not used)
    CONSTANT CMD_USER0      : STD_LOGIC_VECTOR(31 DOWNTO 0) := x"00000006"; 
    -- User command 1 (not used)
    CONSTANT CMD_USER1      : STD_LOGIC_VECTOR(31 DOWNTO 0) := x"00000007";  


  -- command execution signals and registers 
    -- command state machine states
    TYPE cmd_state_type IS( CMD_IDLE,
                            CMD_EXE
                           ); 
    SIGNAL cmd_state        : cmd_state_type; 
    -- new command was received signal
    SIGNAL new_cmd          : STD_LOGIC;       
    -- command register
    SIGNAL cmd_reg          : STD_LOGIC_VECTOR(31 DOWNTO 0);  
    -- fabric input register (from MSS)
    SIGNAL fabin_reg        : STD_LOGIC_VECTOR(31 DOWNTO 0);
    -- fabric output register (to MSS)
    SIGNAL fabout_reg       : STD_LOGIC_VECTOR(31 DOWNTO 0);
    -- fabric status register (to MSS)
    SIGNAL fabst_reg        : STD_LOGIC_VECTOR(31 DOWNTO 0);
    --
    -- multiplexing between daughter boards (0 - HDMI, 1 - SATA connector)
    SIGNAL mux_out          : STD_LOGIC;      
  -- END COMMAND ***************************************************************
  


  -- BEGIN TEMPLATE DESIGN SIGNALS ********************************************* 
  -- state of acquistion register definitions  
    -- acquisition idle state
    CONSTANT OP_IDLE_STATE  : STD_LOGIC_VECTOR(31 DOWNTO 0) := x"FFFFFFFF"; 
    -- busy state
    CONSTANT OP_BUSY_STATE  : STD_LOGIC_VECTOR(31 DOWNTO 0) := x"00000001"; 
    -- buffer overflow state
    CONSTANT OP_OFLW_STATE  : STD_LOGIC_VECTOR(31 DOWNTO 0) := x"00000002"; 
    -- buffer overflow state
    CONSTANT OP_ERR_STATE   : STD_LOGIC_VECTOR(31 DOWNTO 0) := x"00000003"; 
    --
    -- start address of write ram interface operation
    SIGNAL wstart_reg       : STD_LOGIC_VECTOR(31 DOWNTO 0);  
    -- size in bytes of write ram interface operation
    SIGNAL wsize_reg        : STD_LOGIC_VECTOR(31 DOWNTO 0);  
    -- start address of read ram interface operation
    SIGNAL rstart_reg       : STD_LOGIC_VECTOR(31 DOWNTO 0);  
    -- size in bytes of read ram interface operation
    SIGNAL rsize_reg        : STD_LOGIC_VECTOR(31 DOWNTO 0);  
    --
    -- user optional register
    SIGNAL user1_reg        : STD_LOGIC_VECTOR(31 DOWNTO 0);  
    -- user optional register    
    SIGNAL user2_reg        : STD_LOGIC_VECTOR(31 DOWNTO 0);  
    --    
    -- operation state register
    SIGNAL state_reg        : STD_LOGIC_VECTOR(31 DOWNTO 0);  
    --
    -- operation state machine states
    TYPE op_state_type IS     ( OP_IDLE,
                                OP_READ_DATA,
                                OP_OFLW
                              ); 
    SIGNAL op_state     : op_state_type;
    --
    -- operation enable signal
    SIGNAL op_enable    : STD_LOGIC;   
    --              
    -- data input of ram interface
    SIGNAL ram_in       : STD_LOGIC_VECTOR(31 DOWNTO 0); 
    -- valid data input of ram interface
    SIGNAL ram_in_vld   : STD_LOGIC;    
    --
    -- data output of ram interface
    SIGNAL ram_out      : STD_LOGIC_VECTOR(31 DOWNTO 0); 
    -- valid data ouput of ram interface
    SIGNAL ram_out_vld  : STD_LOGIC;    
    --
    -- start address of ram interface operation
    SIGNAL ram_addr     : STD_LOGIC_VECTOR(31 DOWNTO 0); 
    -- number of bytes of ram interface operation           
    SIGNAL ram_cnt      : STD_LOGIC_VECTOR(31 DOWNTO 0); 
    --
    -- ram interface type of operation (1 - write, 0 - read)
    SIGNAL ram_rw       : STD_LOGIC;    
    -- ram interface start signal (assert 1 during one clk period)
    SIGNAL ram_strt     : STD_LOGIC;
    --
    -- read buffer not empty  
    SIGNAL rd_buf_ne    : STD_LOGIC;
    -- write buffer full  
    SIGNAL wr_buf_fl    : STD_LOGIC;
    -- write buffer overflow occurs  
    SIGNAL ram_ofw      : STD_LOGIC;
    -- 
    -- actual used address of ram interface
    SIGNAL stage_reg    : STD_LOGIC_VECTOR(31 DOWNTO 0);
    -- ram interface busy (1 - r/w operation occurs) 
    SIGNAL ram_bsy      : STD_LOGIC;

  -- END TEMPLATE DESIGN SIGNALS ***********************************************

    -- SP converter signals
    SIGNAL sp_data_vld_i: STD_LOGIC;  -- auxiliary data valid signal
    SIGNAL vld_sample   : STD_LOGIC;  -- sample signal of SP converter

BEGIN

  i_ram_if: ram_if 
  PORT MAP (
  -- global signals  
    n_rst      => n_rst,
    clk        => clk,

  -- AHB signals and registers (every is led out to upper entity) --
    htrans     => htrans, 
    hsize      => hsize, 
    hwrite     => hwrite,
    hwdata     => hwdata,
    haddr      => haddr,
    hmastlock  => hmastlock,
    hsel       => hsel,
    hreadyout  => hreadyout,
    hresp      => hresp,
    hrdata     => hrdata,
    hburst     => hburst,
    --
  -- user interface signals --
    data_in      => ram_in,
    data_in_vld  => ram_in_vld,
    --
    data_out     => ram_out,
    data_out_vld => ram_out_vld,
    --
    addr_in    => ram_addr,
    byte_cnt   => ram_cnt,
    --
    data_wr    => ram_rw,
    ram_strt   => ram_strt,
    --
    wr_buf_fl  => wr_buf_fl,
    buff_ofw   => ram_ofw,
    --
    ram_stg    => stage_reg,
    ram_bsy    => ram_bsy
  );

  i_ahb_reg: ahb_reg
  PORT MAP (
  -- global signals  
    n_rst      => n_rst,
    clk        => clk,
    --
  -- AHB SLAVE bus (MSS) 
    haddr_mss  => haddr_mss,
    hwdata_mss => hwdata_mss,
    hrdata_mss => hrdata_mss,
    hsel_mss   => hsel_mss,   
    hready_mss => hready_mss,   
    hwrite_mss => hwrite_mss,
    --
  -- registers available from MSS
    cmd_reg    => cmd_reg,
    state_reg  => state_reg,
    stage_reg  => stage_reg,
    -- registers available from user instructions (UART)
    fabin_reg  => fabin_reg, 
    fabout_reg => fabout_reg,
    fabst_reg  => fabst_reg,
    --
    wstart_reg => wstart_reg,
    wsize_reg  => wsize_reg,
    rstart_reg => rstart_reg,
    rsize_reg  => rsize_reg,
    -- user optional registers
    user1_reg  => user1_reg,
    user2_reg  => user2_reg,
    --
  -- new command received signal (assert high)
    new_cmd    => new_cmd
  );
 

--***************************************************************************--
--*************** COMMAND execution state machine template ******************--
--***************************************************************************--
  PROCESS(clk, n_rst)
  BEGIN
    IF (n_rst = '0') THEN
      cmd_state <= CMD_IDLE;
      mux_out   <= '0';
      sync_en   <= '0';
    ELSIF rising_edge(clk) THEN
      CASE cmd_state IS
        WHEN CMD_IDLE =>              -- command awaiting
          IF (new_cmd = '1') THEN
            cmd_state <= CMD_EXE;
          END IF;
        WHEN CMD_EXE  =>              -- command execution
          CASE cmd_reg IS
            WHEN CMD_OP_ST =>         -- operation is started from MSS
              cmd_state <= CMD_IDLE;  -- by inserting of this command
            WHEN CMD_SETDB1 =>        -- set daughter board 1 as active board
              mux_out   <= '0';       -- (HDMI)
              cmd_state <= CMD_IDLE;
            WHEN CMD_SETDB2 =>        -- set daughter board 2 as active board
              mux_out   <= '1';       -- (SATA)
              cmd_state <= CMD_IDLE;
            WHEN CMD_ENSYNC =>        -- enable synchronization of 32 bit words 
              sync_en   <= '1';       -- loaded from counter
              cmd_state <= CMD_IDLE;
            WHEN CMD_DSSYNC =>        -- disable synchronization of 32 bit words  
              sync_en   <= '0';       -- loaded from counter 
              cmd_state <= CMD_IDLE;
            --
            WHEN CMD_USER0 =>         -- you can insert your own code here
              cmd_state <= CMD_IDLE;  
            WHEN CMD_USER1 =>
              cmd_state <= CMD_IDLE;
            WHEN OTHERS => NULL;
          END CASE;
        WHEN OTHERS => NULL;
      END CASE;
    END IF;
  END PROCESS;

  db_mux_out <= mux_out;  -- multiplexor output for connector choose

--***************************************************************************--


--***************************************************************************--
--************************ USER TEMPLATE DESIGN *****************************--
--***************************************************************************--

  PROCESS(clk, n_rst)
  BEGIN
    IF (n_rst = '0') THEN
      op_state      <= OP_IDLE;
      sp_data_vld_i <= '1';
      ram_in_vld    <= '0';
      ram_strt      <= '0';
    ELSIF rising_edge(clk) THEN
      CASE op_state IS
        WHEN OP_IDLE =>
          IF(op_enable = '1') THEN         -- operation is started by start cmd
            ram_addr      <= wstart_reg;   -- start address of stream
            ram_cnt       <= wsize_reg;    -- size of required stream 
            ram_rw        <= '1';          -- set writing
            -- set default values for valid sampling
            sp_data_vld_i <= '1';
            vld_sample    <= '0';
            -- run ram interface 
            ram_strt      <= '1';
            op_state      <= OP_READ_DATA;
          END IF;    
        WHEN OP_READ_DATA =>
          ram_strt      <= '0';     

          -- if right value of sp_data_vld is set for the second time
          IF((sp_data_vld = sp_data_vld_i) AND (vld_sample = '1'))THEN
            ram_in        <= sp_data;            -- insert data to ram interface
            ram_in_vld    <= '1';
            sp_data_vld_i <= NOT sp_data_vld_i;  -- set new value ov sp_data_vld
            vld_sample    <= '0';                -- reset sp_data_vld sample

          -- if right value of sp_data_vld is set for the first time
          ELSIF((sp_data_vld = sp_data_vld_i) AND (vld_sample = '0'))THEN
            vld_sample    <= '1';                
            ram_in_vld    <= '0';
          ELSE
            ram_in_vld    <= '0';
          END IF;

          
          IF(ram_ofw = '1')THEN                  -- if ram interface overflowed
            op_state <= OP_OFLW;                 -- stay in overflow state
          ELSIF(ram_bsy = '0')THEN               -- when enough of data was loaded
            op_state <= OP_IDLE;                 -- set idle state
          END IF;
        WHEN OP_OFLW =>
          op_state <= OP_OFLW;
      END CASE;
    END IF;
  END PROCESS; 

  -- operation state which is read by MSS
  WITH op_state SELECT 
  state_reg <= OP_IDLE_STATE   WHEN OP_IDLE,
               OP_OFLW_STATE   WHEN OP_OFLW,
               OP_BUSY_STATE   WHEN OTHERS;
 
  -- acquisition enable state
  op_enable <= '1' WHEN ( (cmd_state = CMD_EXE  ) AND 
                          (cmd_reg   = CMD_OP_ST)     ) ELSE '0';
  

  -- not used signals and registers
  fabst_reg    <= (OTHERS =>'0');
  fabout_reg   <= (OTHERS =>'0');
  user2_reg    <= (OTHERS =>'0');
--***************************************************************=************--

END architecture_ctrl;
