-------------------------------------------------------------------------------
--
-- Design unit:   Serial to paralel converter 
--                 
--
-- File name:     s_p_conv.vhd
--
-- Description:   Collect bits to 32-bit words and send it to the next block
--
-- System:        VHDL'93
--
-- Author:        Marek Laban, MICRONIC a.s., LabHC
--
-- Copyright:     MICRONIC a.s., LabHC
--
-- Created in frame of HECTOR project (No. 644052).
--
-- Revision:      Version 1.00, May 2016
-- History:       16/05/2016 - v1.00 Created (ML, MIC)
--
-------------------------------------------------------------------------------


LIBRARY IEEE;

USE ieee.std_logic_unsigned.ALL;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_arith.ALL;

ENTITY s_p_conv IS
  PORT (
  -- Daughter board signal inputs
    clk_in     : IN  STD_LOGIC;    -- clock input
    data_in    : IN  STD_LOGIC;    -- data input signal
    --
    sync_in    : IN  STD_LOGIC;    -- synchronization of 32-bit words
  -- Mother board signals
    n_rst      : IN  STD_LOGIC;    -- reset
    --
    data_vld   : OUT STD_LOGIC;    -- data valid signal (valid on 1 and 0)                 
    -- data output - 32-bit word
    data_out   : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
    -- 
    sync_en    : IN  STD_LOGIC     -- synchronization enable signal  
  );
END s_p_conv;

ARCHITECTURE s_p_conv OF s_p_conv IS
-- internal signal and registers declaration
  SIGNAL cnt_bits : STD_LOGIC_VECTOR(4 DOWNTO 0);  -- bits counter
  SIGNAL vld_i    : STD_LOGIC;                     -- auxilary valid signal
  --
  SIGNAL ser2par  : STD_LOGIC_VECTOR(31 DOWNTO 0); -- fifo bit collecting reg
  SIGNAL data_i   : STD_LOGIC_VECTOR(31 DOWNTO 0); -- data output auxilary reg
  --
  SIGNAL sync     : STD_LOGIC;                     -- synchronization of 32-bit word

BEGIN

-- data collecting
  PROCESS(clk_in, n_rst)
  BEGIN
    IF (n_rst = '0') THEN
      vld_i     <= '0';
      cnt_bits  <= (OTHERS =>'0');
      data_i    <= (OTHERS =>'0');
      ser2par   <= (OTHERS =>'0');
    ELSIF rising_edge(clk_in) THEN  -- clk_in is data strobe signal from TRNG
      IF(cnt_bits = "11111")THEN    -- every 32 bit set new word to output
        vld_i     <= NOT vld_i;     -- invert output strobe signal
        data_i    <= ser2par (30 DOWNTO 0) & data_in;  -- set new word
      ELSE
        ser2par   <= ser2par (30 DOWNTO 0) & data_in;  -- insert new bit
      END IF;

      IF(sync = '1')THEN            -- if synchronization received 
        cnt_bits <= "00000";        -- reset counter
      ELSE
        cnt_bits <= cnt_bits + 1;   -- received bits counter 
      END IF;
    END IF;
  END PROCESS;

  sync     <= sync_en AND sync_in; -- enable 32-bit word syncrhonization
  data_vld <= vld_i;               -- data_valid output
  data_out <= data_i;              -- data output

END s_p_conv;
