-------------------------------------------------------------------------------
--
-- Design unit:   Control block
--                 
--
-- File name:     text_process.vhd
--
-- Description:   Loads 256-bit block of data, analyze every byte of block 
--                and according to txt_format signal, it transforms textsymbols 
--                to lowercase or uppercase format.
--
-- System:        VHDL'93
--
-- Author:        Marek Laban, MICRONIC a.s., LabHC
--
-- Copyright:     MICRONIC a.s., LabHC
--
-- Created in frame of HECTOR project (No. 644052).
--
-- Revision:      Version 1.00, May 2016
-- History:       16/05/2016 - v1.00 Created (ML, MIC)
--
-------------------------------------------------------------------------------
 

LIBRARY IEEE;

USE ieee.std_logic_unsigned.ALL;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_arith.ALL;

ENTITY text_process IS
  PORT (
  -- global signals
    n_rst      : IN STD_LOGIC;
    clk        : IN STD_LOGIC;
  -- communication signals 
    data_out   : OUT STD_LOGIC_VECTOR(255 DOWNTO 0);
    data_in    : IN  STD_LOGIC_VECTOR(255 DOWNTO 0);
    strt_proc  : IN  STD_LOGIC;   -- start process
    end_proc   : OUT STD_LOGIC;   -- process end
    txt_format : IN  STD_LOGIC    -- format of text
  );
END text_process;
ARCHITECTURE architecture_text_process OF text_process IS
    -- letters from ASCII table
    CONSTANT UP_A   : STD_LOGIC_VECTOR(7 DOWNTO 0) := x"41"; 
    CONSTANT UP_Z   : STD_LOGIC_VECTOR(7 DOWNTO 0) := x"5A";
    CONSTANT LOW_A  : STD_LOGIC_VECTOR(7 DOWNTO 0) := x"61";
    CONSTANT LOW_Z  : STD_LOGIC_VECTOR(7 DOWNTO 0) := x"7A";
    -- difference between lowercase and uppercase
    CONSTANT LU_DIF : STD_LOGIC_VECTOR(7 DOWNTO 0) := x"20";
    

    TYPE proc_st_type IS     ( OP_IDLE,
                               OP_ANALYZE,
                               OP_SET_OUT,
                               OP_END
                              ); 
    SIGNAL proc_st             : proc_st_type;

    -- processed data
    SIGNAL proc_data : STD_LOGIC_VECTOR(255 DOWNTO 0); 
    -- counter of processed words
    SIGNAL cnt       : STD_LOGIC_VECTOR(4 DOWNTO 0);   
    -- formated byte
    SIGNAL text_sum  : STD_LOGIC_VECTOR(7 DOWNTO 0);   
    -- format of text (to upper or to lower)
    SIGNAL txt_type  : STD_LOGIC;          
    -- is lowercase signal            
    SIGNAL low_case  : STD_LOGIC;
    -- is uppercase signal 
    SIGNAL up_case   : STD_LOGIC;
    -- case mux output
    SIGNAL word_case : STD_LOGIC;

BEGIN
 
  PROCESS(clk, n_rst)
  BEGIN
    IF (n_rst = '0') THEN
      proc_st   <= OP_IDLE;
      proc_data <= (OTHERS => '0');
      data_out  <= (OTHERS => '0');
      cnt       <= (OTHERS => '0');
      txt_type  <= '0';
      end_proc  <= '0';
    ELSIF rising_edge(clk) THEN
      CASE proc_st IS
        WHEN OP_IDLE =>
          -- wait for start signal              
          IF (strt_proc = '1' ) THEN
            -- save data and format
            proc_data <= data_in;
            txt_type  <= txt_format;
            proc_st   <= OP_ANALYZE;
          END IF;
        WHEN OP_ANALYZE  =>     
          -- if word is uppercase or lowercase insert it to word
          IF (word_case = '1') THEN
            proc_data <= proc_data(247 DOWNTO 0) & (text_sum);
          -- else just shift it
          ELSE
            proc_data <= proc_data(247 DOWNTO 0) & (proc_data(255 DOWNTO 248));
          END IF;

          -- end when 32 words are analyzed
          cnt <= cnt + 1;
          IF (cnt = "11111"  ) THEN
            proc_st  <= OP_SET_OUT;
          END IF;
        WHEN OP_SET_OUT  =>     
          -- set valid data on output and set end process flag
          data_out  <= proc_data;
          end_proc  <= '1';
          proc_st   <= OP_END;
        WHEN OP_END  =>     
          -- wait while start process is not in zero
          IF (strt_proc = '0' ) THEN
            end_proc  <= '0';
            proc_st   <= OP_IDLE;
          END IF;
      END CASE;
    END IF;
  END PROCESS;

  -- analyze of lowercase
  low_case <= '1' WHEN ( ( proc_data(255 DOWNTO 248) >= LOW_A  )   AND 
                         ( proc_data(255 DOWNTO 248) <= LOW_Z  ) ) ELSE '0';
  -- analyze of uppercase
  up_case  <= '1' WHEN ( ( proc_data(255 DOWNTO 248) >= UP_A  )   AND 
                         ( proc_data(255 DOWNTO 248) <= UP_Z  ) ) ELSE '0';

  WITH txt_type SELECT 
  word_case <= low_case WHEN '1',    -- when transformation to uppercase
               up_case  WHEN OTHERS; -- when transformation to lowercase

  -- transformation to configured case
  WITH txt_type SELECT 
  text_sum <= (proc_data(255 DOWNTO 248) - LU_DIF) WHEN '1',    -- when uppercase tran
              (proc_data(255 DOWNTO 248) + LU_DIF) WHEN OTHERS; -- when lowercase tran


END architecture_text_process;
