-------------------------------------------------------------------------------
--
-- Design unit:   HECTOR daughter board example
--                
--
-- File name:     cnt_gen.vhd
--
-- Description:   Simple 32bit counter with 1 bit shift output
--                Designed for Daughter board SmartFusion 2 v1.1 
--                (DBSF2v11)
--
-- System:        VHDL'93
--
-- Author:        Marek Laban, MICRONIC a.s., LabHC
--
-- Copyright:     MICRONIC a.s., LabHC
--
-- Created in frame of HECTOR project (No. 644052).
--
-- Revision:      Version 1.00, May 2016
-- History:       16/05/2016 - v1.00 Created (ML, MIC)
--
-------------------------------------------------------------------------------
LIBRARY IEEE;

USE ieee.std_logic_unsigned.ALL;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_arith.ALL;

ENTITY cnt_gen IS
  
  PORT (
  -- Global clock inputs
    clk      : IN  STD_LOGIC;    -- clock signal
    n_rst    : IN  STD_LOGIC;    -- mother board reset input
  -- Signals to motherboard interface
    data     : OUT  STD_LOGIC;   -- 1 bit of 32-bit word
    data_clk : OUT  STD_LOGIC;   -- valid data
    sync_out : OUT  STD_LOGIC    -- synchronization of 32-bit word
  );
END cnt_gen;

ARCHITECTURE cnt_gen OF cnt_gen IS
  -- counter of sent bits
  SIGNAL cnt_bits      : STD_LOGIC_VECTOR(4 DOWNTO 0);    
  -- register of next word
  SIGNAL cnt           : STD_LOGIC_VECTOR(31 DOWNTO 0);   
  -- fifo register
  SIGNAL par2ser       : STD_LOGIC_VECTOR(31 DOWNTO 0);   
  -- data signal auxiliary signal
  SIGNAL data_clk_i : STD_LOGIC;                
BEGIN
  
  PROCESS(clk, n_rst)
  BEGIN
    IF (n_rst = '0') THEN
      par2ser    <= (OTHERS => '0');
      cnt_bits   <= (OTHERS => '0');
      cnt        <= (OTHERS => '0');
      data_clk_i <= '0';
      sync_out   <= '0';
    ELSIF rising_edge(clk) THEN
      -- every 32 bits set new word
      IF(data_clk_i = '0' AND cnt_bits = "00000")THEN   
        par2ser    <= cnt;
        cnt        <= cnt + 1;
        cnt_bits   <= cnt_bits + 1;
        data_clk_i <= '1';        -- set strobe      
      
      -- else shift bit from fifo
      ELSIF(data_clk_i = '0')THEN 
        par2ser    <= par2ser(30 DOWNTO 0) & '0';
        data_clk_i <= '1';
        cnt_bits   <= cnt_bits + 1;      
      ELSE                       
        data_clk_i <= '0';        -- reset strobe
      END IF;
     
      IF(cnt_bits = "11111")THEN  -- make synchronization to last bit
        sync_out <= '1';
      ELSE
        sync_out <= '0';
      END IF;

    END IF;
  END PROCESS;

  data     <= par2ser(31);        -- shifted bit
  data_clk <= data_clk_i;         -- data strobe signal

END cnt_gen;

 