-------------------------------------------------------------------------------
--
-- Design unit:   HECTOR daughter board example
--                
--
-- File name:     pll_trng.vhd
--
-- Description:   PLL-based TRNG with a possibility to output either
--                the TRNG output bitstream, or raw random data before
--                the XOR decimator.
--                Designed for Daughter board SmartFusion 2 v1.1 
--                (DBSF2v11)
--
-- System:        VHDL'93
--
-- Author:        Oto Petura, LabHC
--
-- Copyright:     Univ Lyon, UJM - LabHC
--
-- Created in frame of HECTOR project (No. 644052).
--
-- Revision:      Version 1.00, May 2016
-- History:       16/05/2016 - v1.00 Created (OP, LabHC)
--
-------------------------------------------------------------------------------

LIBRARY IEEE;

USE IEEE.std_logic_1164.ALL;
USE IEEE.numeric_std.ALL;

ENTITY pll_trng IS
  PORT (
    clk_in     : IN  STD_LOGIC; -- Quartz input
    mux_sel    : IN  STD_LOGIC; -- GPIO0
    n_reset      : IN  STD_LOGIC; -- GPIO2
    data_clk_p : OUT STD_LOGIC; -- LVDS0 P
    data_clk_n : OUT STD_LOGIC; -- LVDS0 N
    data_p     : OUT STD_LOGIC; -- LVDS3 P
    data_n     : OUT STD_LOGIC  -- LVDS3 N
  );
END pll_trng;

ARCHITECTURE arch OF pll_trng IS

  CONSTANT KD : NATURAL := 1305;

  COMPONENT pll1 IS
    -- Port list
    PORT (
      -- Inputs
      XTLOSC : IN  STD_LOGIC;
      -- Outputs
      GL0    : OUT STD_LOGIC;
      LOCK   : OUT STD_LOGIC
    );
  END COMPONENT pll1;

  COMPONENT pll2 IS
    -- Port list
    PORT (
      -- Inputs
      XTLOSC : IN  STD_LOGIC;
      -- Outputs
      GL0    : OUT STD_LOGIC;
      LOCK   : OUT STD_LOGIC
    );
  END COMPONENT pll2;

  COMPONENT lvds_out IS
    -- Port list
    PORT (
      -- Inputs
      D        : IN  STD_LOGIC_VECTOR(0 TO 0);
      -- Outputs
      PADN_OUT : OUT STD_LOGIC_VECTOR(0 TO 0);
      PADP_OUT : OUT STD_LOGIC_VECTOR(0 TO 0)
    );
  END COMPONENT lvds_out;

  COMPONENT xtal is
    -- Port list
    PORT (
      -- Inputs
      XTL        : IN  STD_LOGIC;
      -- Outputs
      XTLOSC_CCC : OUT STD_LOGIC
    );
  END COMPONENT xtal;

  SIGNAL xtal_clk : STD_LOGIC := '0';
  SIGNAL clk      : STD_LOGIC := '0';
  SIGNAL clj      : STD_LOGIC := '0';
  SIGNAL reset    : STD_LOGIC := '0';
  SIGNAL dff      : STD_LOGIC := '0';
  SIGNAL xor_out  : STD_LOGIC := '0';
  SIGNAL cnt      : STD_LOGIC_VECTOR ( 11 DOWNTO 0 ) := ( OTHERS => '0' );
  SIGNAL output   : STD_LOGIC := '0';
  SIGNAL out_clk  : STD_LOGIC := '0';
  SIGNAL rnd_rdy  : STD_LOGIC := '0';

BEGIN

  reset <= NOT n_reset;

  -- XTAL oscillator instance
  xtal_inst : xtal
    PORT MAP(
      XTL        => clk_in,
      XTLOSC_CCC => xtal_clk
    );

  -- PLL1 - generates samplig clock
  pll1_inst : pll1
    PORT MAP (
      XTLOSC => xtal_clk,
      GL0    => clk,
      LOCK   => open
    );

  -- PLL2 - generates sampled ("jittery") clock
  pll2_inst : pll2
    PORT MAP (
      XTLOSC => xtal_clk,
      GL0    => clj,
      LOCK   => open
    );

  -- Sampling DFF
  PROCESS(clk, reset)
  BEGIN
    IF (reset = '1') THEN
      dff <= '0';
    ELSIF rising_edge(clk) THEN
      dff <= clj;
    END IF;
  END PROCESS;

  -- XOR decimator
  PROCESS(clk, reset)
  BEGIN
    IF (reset = '1') THEN
      xor_out <= '0';
    ELSIF rising_edge(clk) THEN
      xor_out <= xor_out XOR dff;
    END IF;
  END PROCESS;

  -- Clock prescaler - generates data ready strobe
  PROCESS(clk, reset)
  BEGIN
    IF (reset = '1') THEN
      cnt     <= (OTHERS => '0');
      rnd_rdy <= '0';
    ELSIF rising_edge(clk) THEN
      IF (UNSIGNED(cnt) = KD) THEN
        cnt     <= (OTHERS => '0');
        rnd_rdy <= '1';
      ELSE
        cnt     <= STD_LOGIC_VECTOR( UNSIGNED( cnt ) + 1 );
        rnd_rdy <= '0';
      END IF;
    END IF;
  END PROCESS;

  -- Output data multiplexer
  WITH mux_sel SELECT
    output <= xor_out WHEN '0',
              dff     WHEN OTHERS;

  -- Strobe output multiplexer
  WITH mux_sel SELECT
    out_clk <= rnd_rdy WHEN '0',
               clk     WHEN OTHERS;

  -- Strobe LVDS output driver
  lvds0_inst : lvds_out
    PORT MAP (
        D(0)        => out_clk,
        PADN_OUT(0) => data_clk_n,
        PADP_OUT(0) => data_clk_p
        );

  -- Data output LVDS driver
  lvds3_inst : lvds_out
    PORT MAP (
        D(0)        => output,
        PADN_OUT(0) => data_n,
        PADP_OUT(0) => data_p
        );


END ARCHITECTURE arch;
